#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>

void *my_memchr(const void *s, int c, size_t n);

int
main (void)
{
  int pagesize;
  char *addr, *result;

  /* Get the memory page size */
  pagesize = sysconf (_SC_PAGESIZE);

  /* Reserve two pages */
  addr = (char *) mmap (NULL, 2 * pagesize, PROT_READ | PROT_WRITE,
			MAP_PRIVATE | MAP_ANON, -1, 0);
  printf ("addr = %p\n", addr);

  /* Fill both pages with 'T' */
  memset (addr, 'T', 2 * pagesize);

  /* Protect the second page */
  mprotect (addr + pagesize, pagesize, PROT_NONE);

  /* Put a 'V' in the last double word of the first page */
  addr[pagesize - 8] = 'V';

  /* Search for the 'V' in both pages. According to C99 and POSIX, the
     operation should stop when either a match is found, or n bytes have
     been scanned. 

     On alpha, the assembly version of memchr() uses prefetching and accesses
     the second page while a match exists in the last double word of the first
     page.
   */
  result = my_memchr (addr, 'V', 2 * pagesize);

  /* Display the resulting address */
  printf ("result = %p\n", result);

  return 0;
}
